@@ -37,7 +37,6 @@ static GeglOptions *opts_new (void)
   o->output   = NULL;
   o->files    = NULL;
   o->file     = NULL;
-  o->delay    = 0.0;
   o->rest     = NULL;
   return o;
 }
@@ -61,7 +60,8 @@ usage (char *application_name)
 "     --dot       output a graphviz graph description\n"
 ""
 "     --output    output generated image to named file\n"
-"     -o          (file is saved in PNG format)\n"
+"     -o          (file is saved in PNG or PPM format\n"
+"                  depending on the file ending.)\n"
 "\n"
 "     -p          (increment frame counters of various elements when\n"
 "                  processing is done.)\n"
@@ -71,12 +71,6 @@ usage (char *application_name)
 "     --verbose   print diagnostics while running\n"
 "      -v\n"
 "\n"
-#if 0
-"     --delay     wait for specified number of seconds before exit\n"
-"                 or number of seconds to wait before iterating the image list\n"
-"     -d\n"
-"\n"
-#endif
 "All parameters following -- are considered ops to be chained together\n"
 "into a small composition instead of using an xml file, this allows for\n"
 "easy testing of filters. Be aware that the default value will be used\n"
@@ -130,14 +124,12 @@ print_opts (GeglOptions *o)
 "\txml:    %s\n"
 "\toutput: %s\n"
 "\trest:   %s\n"
-"\tdelay:  %f\n"
 "\t\n",
     o->mode,
-    o->file,
-    o->xml,
-    o->output,
-    o->rest==NULL?"":"yes",
-    o->delay
+    o->file==NULL?"(null)":o->file,
+    o->xml==NULL?"(null)":o->xml,
+    o->output==NULL?"(null)":o->output,
+    o->rest==NULL?"":"yes"
 );
     {
       GList *files = o->files;
@@ -213,11 +205,6 @@ parse_args (int    argc,
             usage (argv[0]);
         }
 
-        else if (match ("--delay") ||
-                 match ("-d")) {
-            get_float (o->delay);
-        }
-
         else if (match ("--verbose") ||
                  match ("-v")) {
             o->verbose=1;
@@ -246,8 +233,33 @@ parse_args (int    argc,
 
         else if (match ("--output") ||
                  match ("-o")) {
+            const char *trigramstart;
+            int   length;
+
             get_string_forced (o->output);
-            o->mode = GEGL_RUN_MODE_PNG;
+
+            length       = strlen (o->output);
+            trigramstart = o->output;
+
+            /* position the pointer at the start of the trigram or
+             * if the string is not long enough, at the end of the string
+             */
+            trigramstart += (length > 3 ? length - 3 : length);
+
+            if (strcmp (trigramstart, "png") == 0 ||
+                strcmp (trigramstart, "PNG") == 0 ) {
+
+                o->mode = GEGL_RUN_MODE_PNG;
+            } else if (strcmp (trigramstart, "ppm") == 0 ||
+                       strcmp (trigramstart, "PPM") == 0) {
+
+                o->mode = GEGL_RUN_MODE_PPM;
+            } else {
+                fprintf (stderr,
+                         "Output file extension not found (%s), using png\n",
+                         o->output);
+                o->mode = GEGL_RUN_MODE_PNG;
+            }
         }
 
         else if (match ("-X")) {
