@@ -22,6 +22,8 @@
 #include <glib-object.h>
 #include <babl/babl.h>
 
+G_BEGIN_DECLS
+
 #define GEGL_TYPE_BUFFER (gegl_buffer_get_type ())
 #define GEGL_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_BUFFER, GeglBuffer))
 #ifndef __GEGL_BUFFER_TYPES_H__
@@ -295,7 +297,12 @@ typedef enum {
   GEGL_INTERPOLATION_LINEAR,
   GEGL_INTERPOLATION_CUBIC,
   GEGL_INTERPOLATION_LANCZOS,
-  GEGL_INTERPOLATION_YAFR
+  GEGL_INTERPOLATION_DOWNSHARP,
+  GEGL_INTERPOLATION_DOWNSIZE,
+  GEGL_INTERPOLATION_DOWNSMOOTH,
+  GEGL_INTERPOLATION_UPSHARP,
+  GEGL_INTERPOLATION_UPSIZE,
+  GEGL_INTERPOLATION_UPSMOOTH
 } GeglInterpolation;
 
 /**
@@ -419,5 +426,15 @@ void            gegl_buffer_linear_close     (GeglBuffer    *buffer,
 
 
 /**
+ * gegl_buffer_get_abyss:
+ * @buffer: a #GeglBuffer.
+ *
+ * Return the abyss extent of a buffer, this expands out to the parents extent in
+ * subbuffers.
  */
+const GeglRectangle* gegl_buffer_get_abyss  (GeglBuffer           *buffer);
+
+/**
+ */
+G_END_DECLS
 #endif
