@@ -19,7 +19,7 @@
 #include <glib-object.h>
 #include <string.h>
 #include <glib/gprintf.h>
-
+#include "config.h"
 #include "gegl.h"
 #include "gegl-types-internal.h"
 #include "gegl-config.h"
@@ -38,6 +38,9 @@ enum
   PROP_BABL_TOLERANCE,
   PROP_TILE_WIDTH,
   PROP_TILE_HEIGHT
+#if ENABLE_MT
+  ,PROP_THREADS
+#endif
 };
 
 static void
@@ -78,6 +81,12 @@ get_property (GObject    *gobject,
         g_value_set_string (value, config->swap);
         break;
 
+#if ENABLE_MT
+      case PROP_THREADS:
+        g_value_set_int (value, config->threads);
+        break;
+#endif
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
@@ -126,6 +135,11 @@ set_property (GObject      *gobject,
          g_free (config->swap);
         config->swap = g_value_dup_string (value);
         break;
+#if ENABLE_MT
+      case PROP_THREADS:
+        config->threads = g_value_get_int (value);
+        return;
+#endif
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
@@ -190,6 +204,13 @@ gegl_config_class_init (GeglConfigClass *klass)
   g_object_class_install_property (gobject_class, PROP_SWAP,
                                    g_param_spec_string ("swap", "Swap", "where gegl stores it's swap files", NULL,
                                                      G_PARAM_READWRITE));
+
+#if ENABLE_MT
+  g_object_class_install_property (gobject_class, PROP_TILE_HEIGHT,
+                                   g_param_spec_int ("threads", "Number of concurrent evaluation threads", "default tile height for created buffers.",
+                                                     0, 16, 2,
+                                                     G_PARAM_READWRITE));
+#endif
 }
 
 static void
@@ -199,6 +220,9 @@ gegl_config_init (GeglConfig *self)
   self->quality     = 1.0;
   self->cache_size  = 256 * 1024 * 1024;
   self->chunk_size  = 512 * 512;
-  self->tile_width  = 64;
-  self->tile_height = 128;
+  self->tile_width  = 128;
+  self->tile_height = 64;
+#if ENABLE_MT
+  self->threads = 2;
+#endif
 }
