@@ -22,6 +22,15 @@
 #include <glib-object.h>
 #include <babl/babl.h>
 
+#include <gegl-types.h>
+
+#include <gegl-buffer.h>
+#include <gegl-color.h>
+#include <gegl-curve.h>
+#include <gegl-path.h>
+#include <gegl-version.h>
+
+
 /***
  * The GEGL API:
  *
@@ -107,31 +116,6 @@ GOptionGroup * gegl_get_option_group     (void);
 void           gegl_exit                 (void);
 
 
-/***
- * GEGL version information
- *
- * These macros tell the version of GEGL you are compiling against.
- * GEGL's version number consists of three parts: major, minor and
- * micro.
- */
-
-#define GEGL_MAJOR_VERSION 0
-#define GEGL_MINOR_VERSION 0
-#define GEGL_MICRO_VERSION 18
-
-/**
- * gegl_get_version:
- * @major: a pointer to a int where the major version number will be stored
- * @minor: ditto for the minor version number
- * @micro: ditto for the micro version number
- *
- * This function fetches the version of the GEGL library being used by
- * the running process.
- */
-void           gegl_get_version          (int *major,
-                                          int *minor,
-                                          int *micro);
-
 /***
  * Available operations:
  * Gegl provides means to check for available processing operations that
@@ -201,20 +185,6 @@ GParamSpec** gegl_list_properties           (const gchar   *operation_type,
  *                                  "contrast",   1.5,
  *                                  NULL);
  */
-#ifndef GEGL_INTERNAL /* These declarations duplicate internal ones in GEGL */
-
-#ifndef GEGL_TYPE_NODE
-typedef struct _GeglNode  GeglNode;
-GType gegl_node_get_type  (void) G_GNUC_CONST;
-#define GEGL_TYPE_NODE    (gegl_node_get_type())
-#define GEGL_NODE(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_NODE, GeglNode))
-#define GEGL_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_NODE))
-#endif
-
-typedef struct _GeglRectangle GeglRectangle;
-GType gegl_rectangle_get_type (void) G_GNUC_CONST;
-#define GEGL_TYPE_RECTANGLE   (gegl_rectangle_get_type())
-#endif
 
 /**
  * gegl_node_new:
@@ -367,6 +337,19 @@ void          gegl_node_set              (GeglNode      *node,
                                           const gchar   *first_property_name,
                                           ...) G_GNUC_NULL_TERMINATED;
 
+/**
+ * gegl_node_set_valist:
+ * @node: a #GeglNode
+ * @first_property_name: name of the first property to set
+ * @args: value for the first property, followed optionally by more name/value
+ * pairs, followed by NULL.
+ *
+ * valist version of #gegl_node_set
+ */
+void          gegl_node_set_valist       (GeglNode      *node,
+                                          const gchar   *first_property_name,
+                                          va_list        args);
+
 /**
  * gegl_node_get:
  * @node: a #GeglNode
@@ -386,6 +369,19 @@ void          gegl_node_get              (GeglNode      *node,
                                           const gchar   *first_property_name,
                                           ...) G_GNUC_NULL_TERMINATED;
 
+/**
+ * gegl_node_get_valist:
+ * @node: a #GeglNode
+ * @first_property_name: name of the first property to get.
+ * @args: return location for the first property, followed optionally by more
+ * name/value pairs, followed by NULL.
+ *
+ * valist version of #gegl_node_get
+ */
+void          gegl_node_get_valist       (GeglNode      *node,
+                                          const gchar   *first_property_name,
+                                          va_list        args);
+
 
 /***
  * Processing:
@@ -398,20 +394,6 @@ void          gegl_node_get              (GeglNode      *node,
  * sample.
  */
 
-#ifndef GEGL_BLIT_FLAGS
-#define  GEGL_BLIT_FLAGS
-typedef enum
-{
-  GEGL_BLIT_DEFAULT  = 0,
-  GEGL_BLIT_CACHE    = 1 << 0,
-  GEGL_BLIT_DIRTY    = 1 << 1,
-} GeglBlitFlags;
-#endif
-
-#ifndef GEGL_AUTO_ROWSTRIDE
-#define GEGL_AUTO_ROWSTRIDE 0
-#endif
-
 /**
  * gegl_node_blit:
  * @node: a #GeglNode
@@ -485,7 +467,10 @@ void          gegl_node_process          (GeglNode      *sink_node);
  *
  * Sometimes it is useful to be able to move nodes between graphs or even
  * handle orphaned nods that are not part of a graph. gegl_node_adopt_child
- * and gegl_node_get_parent are provided ot handle such cases.
+ * and gegl_node_get_parent are provided to handle such cases.
+ *
+ * (gegl_node_adopt_child is deprecated, and will be removed in a future
+ * release)
  */
 
 /**
@@ -504,6 +489,33 @@ void          gegl_node_process          (GeglNode      *sink_node);
 GeglNode    * gegl_node_adopt_child        (GeglNode      *parent,
                                             GeglNode      *child);
 
+/**
+ * gegl_node_add_child:
+ * @graph: a GeglNode (graph)
+ * @child: a GeglNode.
+ *
+ * Make the GeglNode @graph, take a reference on child. This reference
+ * will be dropped when the reference count on the graph reaches zero.
+ *
+ * Returns the child.
+ */
+GeglNode *    gegl_node_add_child           (GeglNode      *graph,
+                                             GeglNode      *child);
+
+/**
+ * gegl_node_remove_child:
+ * @graph: a GeglNode (graph)
+ * @child: a GeglNode.
+ *
+ * Removes a child from a GeglNode. The reference previously held will be
+ * dropped so increase the reference count before removing when reparenting
+ * a child between two graphs.
+ *
+ * Returns the child.
+ */
+GeglNode *    gegl_node_remove_child        (GeglNode      *graph,
+                                             GeglNode      *child);
+
 /**
  * gegl_node_get_parent:
  * @node: a #GeglNode
@@ -760,13 +772,6 @@ gchar       * gegl_node_to_xml           (GeglNode      *node,
  * should be sufficient. See #gegl_processor_work for a code sample.
  *
  */
-#ifndef GEGL_INTERNAL
-typedef struct _GeglProcessor  GeglProcessor;
-GType gegl_processor_get_type  (void) G_GNUC_CONST;
-#define GEGL_TYPE_PROCESSOR    (gegl_processor_get_type())
-#define GEGL_PROCESSOR(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_PROCESSOR, GeglProcessor))
-#define GEGL_IS_PROCESSOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_PROCESSOR))
-#endif
 
 /**
  * gegl_node_new_processor:
@@ -820,62 +825,62 @@ gboolean       gegl_processor_work          (GeglProcessor *processor,
  */
 void           gegl_processor_destroy       (GeglProcessor *processor);
 
+
+/***
+ * GeglConfig:
+ *
+ * GEGL uses a singleton configuration object
+ */
+
 /**
  * gegl_config:
  *
- * Returns a GObject with properties that can be manipulated to control
+ * Returns a GeglConfig object with properties that can be manipulated to control
  * GEGLs behavior. Properties available on the object are:
  *
  * "cache-size" "quality" and "swap", the two first is an integer denoting
  * number of bytes, the secons a double value between 0 and 1 and the last
  * the path of the directory to swap to (or "ram" to not use diskbased swap)
  */
-GObject      * gegl_config (void);
+GeglConfig      * gegl_config (void);
 
 
-#ifndef GEGL_INTERNAL
-/***
- * GeglRectangle:
+/**
+ * gegl_node:
+ * @op_type:  the type of operation to create
+ * @first_property_name: 
+ * @...:
  *
- * GeglRectangles are used in #gegl_node_get_bounding_box and #gegl_node_blit
- * for specifying rectangles.
+ * Construct a GEGL node, connecting it to needed input nodes. The
+ * returned node does not have a parent but a single reference it
+ * is meant to be passed to gegl_graph () for gegl_graph () to assume
+ * its ownership. This is syntactic sugar for use from C, similar
+ * conveniences can easily be built externally in other languages.
  *
- * </p><pre>struct GeglRectangle
- * {
- *   gint x;
- *   gint y;
- *   gint width;
- *   gint height;
- * };</pre><p>
+ * gegl_node(op_type, [key, value, [...]], NULL, [input, [aux]])
  *
+ * Returns a new Gegl node.
  */
-struct _GeglRectangle
-{
-  gint x;
-  gint y;
-  gint width;
-  gint height;
-};
-
-#define GEGL_PAD_TYPE
-
-typedef enum
-{
-  GEGL_PARAM_PAD_OUTPUT = 1 << G_PARAM_USER_SHIFT,
-  GEGL_PARAM_PAD_INPUT  = 1 << (G_PARAM_USER_SHIFT + 1)
-} GeglPadType;
-
-typedef struct _GeglCurve  GeglCurve;
-typedef struct _GeglVector  GeglVector;
-typedef struct _GeglColor  GeglColor;
-#endif
+GeglNode *gegl_node (const gchar *op_type,
+                     const gchar *first_property_name,
+                     ...);
 
 
-/*** foo
+/**
+ * gegl_graph:
+ * @node: the end result of a composition created with gegl_node()
+ *
+ * Creates a GeglNode containing a free floating graph constructed
+ * using gegl_node(). The GeglGraph adopts all the passed in nodes
+ * making it sufficient to unref the resulting graph.
+ *
+ * gegl_graph (gegl_node ("gegl:over", NULL,
+ *                        gegl_node (..), gegl_node (..)));
+ *
+ * Returns a GeglNode graph.
  */
-#include <gegl-buffer.h>
-#include <gegl-color.h>
-#include <gegl-curve.h>
+GeglNode *gegl_graph (GeglNode *node);
+
 
 G_END_DECLS
 #endif  /* __GEGL_H__ */
