@@ -178,7 +178,6 @@ set_property (GObject      *object,
     }
 }
 
-
 static gboolean
 gegl_operation_filter_process (GeglOperation   *operation,
                                GeglOperationContext *context,
@@ -204,27 +203,14 @@ gegl_operation_filter_process (GeglOperation   *operation,
 
   input  = gegl_operation_context_get_source (context, "input");
   output = gegl_operation_context_get_target (context, "output");
-    {
-      success = klass->process (operation, input, output, result);
 
-      if (output == GEGL_BUFFER (operation->node->cache))
-        gegl_cache_computed (operation->node->cache, result);
+  success = klass->process (operation, input, output, result);
 
-      if (input != NULL)
-        g_object_unref (input);
-    }
-#if 0
-  else
-    {
-      /* if we have the data "graph" associated" we're a proxy-nop, and thus
-       * we might legitimatly get NULL (at least layers might)
-       */
-      if (!g_object_get_data (G_OBJECT (operation->node), "graph"))
-        g_warning ("%s got %s %s",
-                   gegl_node_get_debug_name (operation->node),
-                   input==NULL?"input==NULL":"", output==NULL?"output==NULL":"");
-    }
-#endif
+  if (output == GEGL_BUFFER (operation->node->cache))
+    gegl_cache_computed (operation->node->cache, result);
+
+  if (input != NULL)
+    g_object_unref (input);
   return success;
 }
 
