@@ -185,15 +185,17 @@ gegl_path_list_append_item  (GeglPathList  *head,
 
   if (iter)
     {
+      /* the +3 is padding, +1 was excpected to be sufficient */
       iter->next = 
-        g_slice_alloc0 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 *(info->n_items+1)/2);
+        g_slice_alloc0 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 *(info->n_items+3)/2);
       iter->next->d.type = type;
       iter = iter->next;
     }
   else /* creating new path */
     {
+      /* the +3 is padding, +1 was excpected to be sufficient */
       head = 
-        g_slice_alloc0 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 *(info->n_items+1)/2);
+        g_slice_alloc0 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 *(info->n_items+3)/2);
       head->d.type = type;
       iter=head;
     }
@@ -1353,7 +1355,7 @@ gegl_path_get_node (GeglPath     *vector,
 static void gegl_path_item_free (GeglPathList *p)
 {
   InstructionInfo *info = lookup_instruction_info(p->d.type);
-  g_slice_free1 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 * (info->n_items+1)/2, p);
+  g_slice_free1 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 * (info->n_items+3)/2, p);
 }
 
 void  gegl_path_remove_node  (GeglPath *vector,
@@ -1404,7 +1406,7 @@ void  gegl_path_insert_node     (GeglPath           *vector,
     {
       if (count == pos)
         {
-          GeglPathList *new = g_slice_alloc0 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 *(info->n_items+1)/2);
+          GeglPathList *new = g_slice_alloc0 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 *(info->n_items+3)/2);
           new->d.type=knot->type;
           copy_data (knot, &new->d);
           new->next = iter->next;
@@ -1421,7 +1423,7 @@ void  gegl_path_insert_node     (GeglPath           *vector,
     }
   if (pos==-1)
     {
-      GeglPathList *new = g_slice_alloc0 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 *(info->n_items+1)/2);
+      GeglPathList *new = g_slice_alloc0 (sizeof (gpointer) + sizeof (gchar) + sizeof (gfloat)*2 *(info->n_items+3)/2);
       new->d.type = knot->type;
       copy_data (knot, &new->d);
       new->next = NULL;
@@ -1890,8 +1892,10 @@ fill_close:  /* label used for goto to close last segment */
     col[2] *= factor;
     col[3] *= factor;
 
+#if 0
     if (gegl_buffer_is_shared (buffer))
     while (!gegl_buffer_try_lock (buffer));
+#endif
 
     for (i=0; i < extent.height * versub; i++)
       {
@@ -1993,7 +1997,7 @@ static void gegl_path_stamp (GeglBuffer *buffer,
     }
   g_assert (s.buf);
 
-  gegl_buffer_get (buffer, 1.0, &roi, s.format, s.buf, 0);
+  gegl_buffer_get_unlocked (buffer, 1.0, &roi, s.format, s.buf, 0);
 
   {
     gint u, v;
@@ -2031,7 +2035,7 @@ static void gegl_path_stamp (GeglBuffer *buffer,
         }
     }
   }
-  gegl_buffer_set (buffer, &roi, s.format, s.buf, 0);
+  gegl_buffer_set_unlocked (buffer, &roi, s.format, s.buf, 0);
 }
 
 
@@ -2164,7 +2168,7 @@ void gegl_path_stroke (GeglBuffer *buffer,
     }
 
   if (gegl_buffer_is_shared (buffer))
-  gegl_buffer_unlock (buffer);
+    gegl_buffer_unlock (buffer);
 }
 
 gint gegl_path_type_get_n_items (gchar type)
