@@ -28,7 +28,9 @@
 
 
 #include <math.h>
+#include <gegl.h>
 #include <gegl-plugin.h>
+#include <gegl-utils.h>
 #include "buffer/gegl-sampler.h"
 #include <graph/gegl-pad.h>
 #include <graph/gegl-node.h>
@@ -84,6 +86,19 @@ static GeglNode    * gegl_affine_detect                    (GeglOperation
                                                             gint                  x,
                                                             gint                  y);
 
+static gboolean      gegl_affine_matrix3_allow_fast_translate      (GeglMatrix3 matrix);
+static gboolean      gegl_affine_matrix3_allow_fast_reflect_x      (GeglMatrix3 matrix);
+static gboolean      gegl_affine_matrix3_allow_fast_reflect_y      (GeglMatrix3 matrix);
+
+static void          gegl_affine_fast_reflect_x            (GeglBuffer           *dest,
+                                                            GeglBuffer           *src,
+                                                            const GeglRectangle  *dest_rect,
+                                                            const GeglRectangle  *src_rect);
+static void          gegl_affine_fast_reflect_y            (GeglBuffer           *dest,
+                                                            GeglBuffer           *src,
+                                                            const GeglRectangle  *dest_rect,
+                                                            const GeglRectangle  *src_rect);
+
 
 /* ************************* */
 
@@ -132,8 +147,8 @@ GType
 gegl_sampler_type_from_interpolation (GeglInterpolation interpolation);
 
 /* ************************* */
-static void
-op_affine_sampler_init (OpAffine *self)
+static GeglSampler *
+op_affine_sampler (OpAffine *self)
 {
   Babl                 *format;
   GeglSampler          *sampler;
@@ -145,6 +160,33 @@ op_affine_sampler_init (OpAffine *self)
   interpolation = gegl_buffer_interpolation_from_string (self->filter);
   desired_type = gegl_sampler_type_from_interpolation (interpolation);
 
+  if (interpolation == GEGL_INTERPOLATION_LANCZOS)
+    {
+      sampler = g_object_new (desired_type,
+                              "format", format,
+                              "lanczos_width",  self->lanczos_width,
+                              NULL);
+    }
+  else
+    {
+      sampler = g_object_new (desired_type,
+                              "format", format,
+                              NULL);
+    }
+  return sampler;
+}
+/* XXX: keep a pool of samplers */
+
+#if 0
+static void
+op_affine_sampler_init (OpAffine *self)
+{
+  GType                 desired_type;
+  GeglInterpolation     interpolation;
+
+  interpolation = gegl_buffer_interpolation_from_string (self->filter);
+  desired_type = gegl_sampler_type_from_interpolation (interpolation);
+
   if (self->sampler != NULL &&
       !G_TYPE_CHECK_INSTANCE_TYPE (self->sampler, desired_type))
     {
@@ -153,47 +195,20 @@ op_affine_sampler_init (OpAffine *self)
       self->sampler = NULL;
     }
 
-  if (self->sampler == NULL)
-    {
-      if (interpolation == GEGL_INTERPOLATION_LANCZOS)
-        {
-          sampler = g_object_new (desired_type,
-                                  "format", format,
-                                  "lanczos_width",  self->lanczos_width,
-                                  NULL);
-        }
-      else
-        {
-          sampler = g_object_new (desired_type,
-                                  "format", format,
-                                  NULL);
-        }
-      self->sampler = g_object_ref(sampler);
-    }
+  self->sampler = op_affine_sampler (self);
 }
+#endif
 
 static void
 gegl_affine_prepare (GeglOperation *operation)
 {
-  OpAffine  *affine = (OpAffine *) operation;
   Babl      *format = babl_format ("RaGaBaA float");
-  op_affine_sampler_init (affine);
+  /*op_affine_sampler_init (affine);*/
   /*gegl_operation_set_format (operation, "input", format);
   gegl_operation_set_format (operation, "aux", format); XXX(not used yet) */
   gegl_operation_set_format (operation, "output", format);
 }
 
-static void
-gegl_affine_finalize (GObject *object)
-{
-  OpAffine  *affine = (OpAffine *) object;
-  if (affine->sampler != NULL)
-    {
-      g_object_unref(affine->sampler);
-      affine->sampler = NULL;
-    }
-}
-
 static void
 op_affine_class_init (OpAffineClass *klass)
 {
@@ -203,7 +218,6 @@ op_affine_class_init (OpAffineClass *klass)
 
   gobject_class->set_property         = gegl_affine_set_property;
   gobject_class->get_property         = gegl_affine_get_property;
-  gobject_class->finalize             = gegl_affine_finalize;
 
   op_class->get_invalidated_by_change = gegl_affine_get_invalidated_by_change;
   op_class->get_bounding_box          = gegl_affine_get_bounding_box;
@@ -238,7 +252,7 @@ op_affine_class_init (OpAffineClass *klass)
                                    g_param_spec_string (
                                      "filter",
                                      _("Filter"),
-                                     _("Filter type (nearest, linear, lanczos, cubic)"),
+                                     _("Filter type (nearest, linear, lanczos, cubic, downsharp, downsize, downsmooth, upsharp, upsize, upsmooth)"),
                                      "linear",
                                      G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_HARD_EDGES,
@@ -433,6 +447,7 @@ gegl_affine_get_bounding_box (GeglOperation *op)
 {
   OpAffine      *affine  = (OpAffine *) op;
   OpAffineClass *klass   = OP_AFFINE_GET_CLASS (affine);
+  GeglMatrix3    matrix;
   GeglRectangle  in_rect = {0,0,0,0},
                  have_rect;
   gdouble        have_points [8];
@@ -441,31 +456,30 @@ gegl_affine_get_bounding_box (GeglOperation *op)
   GeglRectangle  context_rect;
   GeglSampler   *sampler;
 
-  op_affine_sampler_init (affine);
-  sampler = affine->sampler;
+  sampler = op_affine_sampler (OP_AFFINE (op));
   context_rect = sampler->context_rect;
-
+  g_object_unref (sampler);
 
   if (gegl_operation_source_get_bounding_box (op, "input"))
     in_rect = *gegl_operation_source_get_bounding_box (op, "input");
 
   /* invoke child's matrix creation function */
   g_assert (klass->create_matrix);
-  gegl_matrix3_identity (affine->matrix);
-  klass->create_matrix (op, affine->matrix);
+  gegl_matrix3_identity (matrix);
+  klass->create_matrix (op, matrix);
 
   if (affine->origin_x || affine->origin_y)
-    gegl_matrix3_originate (affine->matrix, affine->origin_x, affine->origin_y);
+    gegl_matrix3_originate (matrix, affine->origin_x, affine->origin_y);
 
   if (gegl_affine_is_composite_node (affine))
     {
       GeglMatrix3 source;
 
       gegl_affine_get_source_matrix (affine, source);
-      gegl_matrix3_multiply (source, affine->matrix, affine->matrix);
+      gegl_matrix3_multiply (source, matrix, matrix);
     }
   if (gegl_affine_is_intermediate_node (affine) ||
-      gegl_matrix3_is_identity (affine->matrix))
+      gegl_matrix3_is_identity (matrix))
     {
       return in_rect;
     }
@@ -542,8 +556,9 @@ gegl_affine_get_required_for_output (GeglOperation       *op,
   gint           i;
 
   requested_rect = *region;
-  sampler = affine->sampler;
+  sampler = op_affine_sampler (OP_AFFINE (op));
   context_rect = sampler->context_rect;
+  g_object_unref (sampler);
 
   gegl_matrix3_copy (inverse, affine->matrix);
   gegl_matrix3_invert (inverse);
@@ -595,9 +610,9 @@ gegl_affine_get_invalidated_by_change (GeglOperation       *op,
   gint               i;
   GeglRectangle      region = *input_region;
 
-  op_affine_sampler_init (affine);
-  sampler = affine->sampler;
+  sampler = op_affine_sampler (OP_AFFINE (op));
   context_rect = sampler->context_rect;
+  g_object_unref (sampler);
   /* invoke child's matrix creation function */
   g_assert (klass->create_matrix);
   gegl_matrix3_identity (affine->matrix);
@@ -658,6 +673,7 @@ affine_generic (GeglBuffer  *dest,
   gfloat               *dest_buf,
                        *dest_ptr;
   GeglMatrix3           inverse;
+  GeglMatrix2           inverse_jacobian;
   gdouble               u_start,
                         v_start,
                         u_float,
@@ -685,6 +701,13 @@ affine_generic (GeglBuffer  *dest,
 
       gegl_matrix3_copy (inverse, matrix);
       gegl_matrix3_invert (inverse);
+      inverse_jacobian[0][0] = inverse[0][0];
+      inverse_jacobian[0][1] = inverse[0][1];
+      inverse_jacobian[1][0] = inverse[1][0];
+      inverse_jacobian[1][1] = inverse[1][1];
+
+     /* set inverse_jacobian for samplers that support it */
+      sampler->inverse_jacobian = &inverse_jacobian;
 
       u_start = inverse[0][0] * roi->x + inverse[0][1] * roi->y + inverse[0][2];
       v_start = inverse[1][0] * roi->x + inverse[1][1] * roi->y + inverse[1][2];
@@ -711,6 +734,120 @@ affine_generic (GeglBuffer  *dest,
     }
 }
 
+static gboolean
+gegl_affine_matrix3_allow_fast_translate (GeglMatrix3 matrix)
+{
+  if (! GEGL_FLOAT_EQUAL (matrix[0][2], (gint) matrix[0][2]) ||
+      ! GEGL_FLOAT_EQUAL (matrix[1][2], (gint) matrix[1][2]))
+    return FALSE;
+  return gegl_matrix3_is_translate (matrix);
+}
+
+static gboolean
+gegl_affine_matrix3_allow_fast_reflect_x (GeglMatrix3 matrix)
+{
+  GeglMatrix3 copy;
+
+  if (! GEGL_FLOAT_EQUAL (matrix[1][1], -1.0))
+    return FALSE;
+  gegl_matrix3_copy (copy, matrix);
+  copy[1][1] = 1.;
+  return gegl_affine_matrix3_allow_fast_translate (copy);
+}
+
+static gboolean
+gegl_affine_matrix3_allow_fast_reflect_y (GeglMatrix3 matrix)
+{
+  GeglMatrix3 copy;
+
+  if (! GEGL_FLOAT_EQUAL (matrix[0][0], -1.0))
+    return FALSE;
+  gegl_matrix3_copy (copy, matrix);
+  copy[0][0] = 1.;
+  return gegl_affine_matrix3_allow_fast_translate (copy);
+}
+
+static void
+gegl_affine_fast_reflect_x (GeglBuffer              *dest,
+                            GeglBuffer              *src,
+                            const GeglRectangle     *dest_rect,
+                            const GeglRectangle     *src_rect)
+{
+  const Babl              *format = gegl_buffer_get_format (src);
+  const gint               px_size = babl_format_get_bytes_per_pixel (format),
+                           rowstride = src_rect->width * px_size;
+  gint                     i;
+  guchar                  *buf = (guchar *) g_malloc (src_rect->height * rowstride);
+
+  gegl_buffer_get (src, 1.0, src_rect, format, buf, GEGL_AUTO_ROWSTRIDE);
+
+  for (i = 0; i < src_rect->height / 2; i++)
+    {
+      gint      dest_offset = (src_rect->height - i - 1) * rowstride,
+                src_offset = i * rowstride,
+                j;
+
+      for (j = 0; j < rowstride; j++)
+        {
+          const guchar      tmp = buf[src_offset];
+
+          buf[src_offset] = buf[dest_offset];
+          buf[dest_offset] = tmp;
+
+          dest_offset++;
+          src_offset++;
+        }
+    }
+
+  gegl_buffer_set (dest, dest_rect, format, buf, GEGL_AUTO_ROWSTRIDE);
+  g_free (buf);
+}
+
+static void
+gegl_affine_fast_reflect_y (GeglBuffer              *dest,
+                            GeglBuffer              *src,
+                            const GeglRectangle     *dest_rect,
+                            const GeglRectangle     *src_rect)
+{
+  const Babl              *format = gegl_buffer_get_format (src);
+  const gint               px_size = babl_format_get_bytes_per_pixel (format),
+                           rowstride = src_rect->width * px_size;
+  gint                     i;
+  guchar                  *buf = (guchar *) g_malloc (src_rect->height * rowstride);
+
+  gegl_buffer_get (src, 1.0, src_rect, format, buf, GEGL_AUTO_ROWSTRIDE);
+
+  for (i = 0; i < src_rect->height; i++)
+    {
+      gint      src_offset = i * rowstride,
+                dest_offset = src_offset + rowstride,
+                j;
+
+      for (j = 0; j < src_rect->width / 2; j++)
+        {
+          gint k;
+
+          dest_offset -= px_size;
+
+          for (k = 0; k < px_size; k++)
+            {
+              const guchar      tmp = buf[src_offset];
+
+              buf[src_offset] = buf[dest_offset];
+              buf[dest_offset] = tmp;
+
+              dest_offset++;
+              src_offset++;
+            }
+
+          dest_offset -= px_size;
+        }
+    }
+
+  gegl_buffer_set (dest, dest_rect, format, buf, GEGL_AUTO_ROWSTRIDE);
+  g_free (buf);
+}
+
 void  gegl_sampler_prepare     (GeglSampler *self);
   /*XXX: Eeeek, obsessive avoidance of public headers, the API needed to
    *     satisfy this use case should probably be provided.
@@ -739,10 +876,9 @@ gegl_affine_process (GeglOperation        *operation,
 
       gegl_operation_context_take_object (context, "output", G_OBJECT (input));
     }
-  else if (gegl_matrix3_is_translate (affine->matrix) &&
-           (! strcmp (affine->filter, "nearest") ||
-            (affine->matrix [0][2] == (gint) affine->matrix [0][2] &&
-             affine->matrix [1][2] == (gint) affine->matrix [1][2])))
+  else if (gegl_affine_matrix3_allow_fast_translate (affine->matrix) ||
+           (gegl_matrix3_is_translate (affine->matrix) &&
+            ! strcmp (affine->filter, "nearest")))
     {
       /* doing a buffer shifting trick, (enhanced nop) */
       input  = gegl_operation_context_get_source (context, "input");
@@ -756,23 +892,81 @@ gegl_affine_process (GeglOperation        *operation,
                                                     of source) */
                          NULL);
 
+      if (gegl_object_get_has_forked (input))
+        gegl_object_set_has_forked (output);
+
       gegl_operation_context_take_object (context, "output", G_OBJECT (output));
 
+      if (input != NULL)
+        g_object_unref (input);
+    }
+  else if (gegl_affine_matrix3_allow_fast_reflect_x (affine->matrix))
+    {
+      GeglRectangle      src_rect;
+      GeglSampler       *sampler;
+
+      input  = gegl_operation_context_get_source (context, "input");
+      if (!input)
+        {
+          g_warning ("transform received NULL input");
+          return FALSE;
+        }
+
+      output = gegl_operation_context_get_target (context, "output");
+
+      src_rect = gegl_operation_get_required_for_output (operation, "output", result);
+      src_rect.y += 1;
+
+      sampler = op_affine_sampler (OP_AFFINE (operation));
+      src_rect.width -= sampler->context_rect.width;
+      src_rect.height -= sampler->context_rect.height;
+
+      gegl_affine_fast_reflect_x (output, input, result, &src_rect);
+
+      if (input != NULL)
+        g_object_unref (input);
+    }
+  else if (gegl_affine_matrix3_allow_fast_reflect_y (affine->matrix))
+    {
+      GeglRectangle      src_rect;
+      GeglSampler       *sampler;
+
+      input  = gegl_operation_context_get_source (context, "input");
+      if (!input)
+        {
+          g_warning ("transform received NULL input");
+          return FALSE;
+        }
+
+      output = gegl_operation_context_get_target (context, "output");
+
+      src_rect = gegl_operation_get_required_for_output (operation, "output", result);
+      src_rect.x += 1;
+
+      sampler = op_affine_sampler (OP_AFFINE (operation));
+      src_rect.width -= sampler->context_rect.width;
+      src_rect.height -= sampler->context_rect.height;
+
+      gegl_affine_fast_reflect_y (output, input, result, &src_rect);
+
       if (input != NULL)
         g_object_unref (input);
     }
   else
     {
       /* for all other cases, do a proper resampling */
+      GeglSampler *sampler;
 
       input  = gegl_operation_context_get_source (context, "input");
       output = gegl_operation_context_get_target (context, "output");
 
-      g_object_set(affine->sampler, "buffer", input, NULL);
-      gegl_sampler_prepare (affine->sampler);
-      affine_generic (output, input, affine->matrix, affine->sampler);
-      g_object_unref(affine->sampler->buffer);
-      affine->sampler->buffer = NULL;
+      sampler = op_affine_sampler (affine);
+      g_object_set(sampler, "buffer", input, NULL);
+      gegl_sampler_prepare (sampler);
+      affine_generic (output, input, affine->matrix, sampler);
+      g_object_unref(sampler->buffer);
+      sampler->buffer = NULL;
+      g_object_unref (sampler);
 
       if (input != NULL)
         g_object_unref (input);
