@@ -72,7 +72,9 @@ load_buffer (GeglChantO *op_raw_load)
         }
 
         {
-          GeglRectangle extent = { 0, 0, width, height };
+          GeglRectangle extent = { 0, 0, 0, 0 };
+          extent.width = width;
+          extent.height = height;
           op_raw_load->chant_data = (gpointer) gegl_buffer_new (&extent,
                                                    babl_format_new (
                                                      babl_model ("RGB"),
@@ -134,7 +136,7 @@ process (GeglOperation       *operation,
   g_assert (g_str_equal (output_pad, "output"));
 
   output = GEGL_BUFFER (o->chant_data);
-  gegl_operation_context_set_object (context, "output", G_OBJECT (output));
+  gegl_operation_context_take_object (context, "output", G_OBJECT (output));
 
   o->chant_data = NULL;
   return TRUE;
