@@ -59,7 +59,7 @@ typedef struct
 
 #define GEGL_CHANT_C_FILE "rectangle.c"
 #include "gegl-chant.h"
-GEGL_DEFINE_DYNAMIC_OPERATION(GEGL_TYPE_OPERATION_META);
+GEGL_DEFINE_DYNAMIC_OPERATION(GEGL_TYPE_OPERATION_META)
 
 static void attach (GeglOperation *operation)
 {
