@@ -137,7 +137,12 @@ gegl_crop_process (GeglOperation        *operation,
       GeglBuffer *output;
 
       output = gegl_buffer_create_sub_buffer (input, &extent);
+
+      if (gegl_object_get_has_forked (input))
+        gegl_object_set_has_forked (output);
+
       gegl_operation_context_take_object (context, "output", G_OBJECT (output));
+
       g_object_unref (input);
       success = TRUE;
     }
