@@ -88,7 +88,12 @@ instead.
     if (pixbuf)
     {
       guchar        *pixeldata;
-      GeglRectangle  rect = {dest_x, dest_y, width, height};
+      GeglRectangle  rect;
+
+      rect.x = dest_x;
+      rect.y = dest_y;
+      rect.width = width;
+      rect.height = height;
 
       pixeldata = gdk_pixbuf_get_pixels (pixbuf);
       gegl_buffer_set (gegl_buffer, &rect, babl_format ("R'G'B'A u8"), pixeldata, GEGL_AUTO_ROWSTRIDE);
@@ -238,14 +243,14 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->categories  = "input";   /* not hidden because it has extra API */
   operation_class->description = _("Load an SVG file using librsvg");
 
-//  static gboolean done=FALSE;
-//  if (done)
-//    return;
+/*  static gboolean done=FALSE;
+    if (done)
+      return; */
   gegl_extension_handler_register (".svg", "gegl:svg-load");
   gegl_extension_handler_register (".SVG", "gegl:svg-load");
   gegl_extension_handler_register (".svgz", "gegl:svg-load");
   gegl_extension_handler_register (".SVGZ", "gegl:svg-load");
-//  done = TRUE;
+/*  done = TRUE; */
 }
 
 #endif
