@@ -17,8 +17,13 @@
  */
 
 #include "config.h"
+
+#include <stdlib.h>
+
 #include <glib/gi18n-lib.h>
 
+#include <libswscale/swscale.h>
+
 
 #ifdef GEGL_CHANT_PROPERTIES
 
@@ -32,7 +37,12 @@ gegl_chant_double (fps, _("FPS"), 0.0, 100.0, 25, _("frames per second"))
 #define GEGL_CHANT_C_FILE       "ff-save.c"
 
 #include "gegl-chant.h"
-#include "ffmpeg/avformat.h"
+
+#ifdef HAVE_LIBAVFORMAT_AVFORMAT_H
+#include <libavformat/avformat.h>
+#else
+#include <avformat.h>
+#endif
 
 typedef struct
 {
@@ -414,7 +424,7 @@ open_audio (Priv * p, AVFormatContext * oc, AVStream * st)
     {
       p->audio_input_frame_size = c->frame_size;
     }
-  //audio_input_frame_size = 44100/25;
+  /*audio_input_frame_size = 44100/25;*/
   p->samples = malloc (p->audio_input_frame_size * 2 * c->channels);
 }
 
@@ -429,7 +439,7 @@ write_audio_frame (GeglChantO *op, AVFormatContext * oc, AVStream * st)
 
   c = st->codec;
 
-  //fprintf (stderr, "going to grab %i\n", p->fragment_size);
+  /*fprintf (stderr, "going to grab %i\n", p->fragment_size);*/
   if (p->oxide_audio_get_fragment (p->oxide_audio_instance,
                                    p->fragment) == (signed) p->fragment_size)
     {
@@ -457,7 +467,7 @@ write_audio_frame (GeglChantO *op, AVFormatContext * oc, AVStream * st)
     }
 }
 
-//p->audio_get_frame (samples, audio_input_frame_size, c->channels);
+/*p->audio_get_frame (samples, audio_input_frame_size, c->channels);*/
 
 void
 close_audio (Priv * p, AVFormatContext * oc, AVStream * st)
@@ -640,14 +650,31 @@ write_video_frame (GeglChantO *op,
 
   if (c->pix_fmt != PIX_FMT_RGB24)
     {
+      struct SwsContext *img_convert_ctx;
+
       /* as we only generate a RGB24 picture, we must convert it
          to the codec pixel format if needed */
       fill_yuv_image (op, p->tmp_picture, p->frame_count, c->width,
                       c->height);
-      /* FIXME: img_convert is deprecated. Update code to use sws_scale(). */
-      img_convert ((AVPicture *) p->picture, c->pix_fmt,
-                   (AVPicture *) p->tmp_picture, PIX_FMT_RGB24,
-                   c->width, c->height);
+
+      img_convert_ctx = sws_getContext(c->width, c->height, c->pix_fmt,
+                                       c->width, c->height, PIX_FMT_RGB24,
+                                       SWS_BICUBIC, NULL, NULL, NULL);
+
+      if (img_convert_ctx == NULL)
+        {
+          fprintf(stderr, "ff_save: Cannot initialize conversion context.");
+        }
+      else
+        {
+          sws_scale(img_convert_ctx,
+                    p->tmp_picture->data,
+                    p->tmp_picture->linesize,
+                    0,
+                    c->height,
+                    p->picture->data,
+                    p->picture->linesize);
+        }
     }
   else
     {
@@ -739,7 +766,7 @@ tfile (GeglChantO *self)
     }
   if (p->oxide_audio_query && p->fmt->audio_codec != CODEC_ID_NONE)
     {
-     //XXX: FOO p->audio_st = add_audio_stream (op, p->oc, p->fmt->audio_codec);
+     /*XXX: FOO p->audio_st = add_audio_stream (op, p->oc, p->fmt->audio_codec);*/
     }
 
   if (av_set_parameters (p->oc, NULL) < 0)
@@ -855,7 +882,7 @@ gegl_chant_class_init (GeglChantClass *klass)
   sink_class->process = process;
   sink_class->needs_full = TRUE;
 
-  operation_class->name        = "ff-save";
+  operation_class->name        = "gegl:ff-save";
   operation_class->categories  = "output:video";
   operation_class->description = _("FFmpeg video output sink");
 }
